*=====================================================================*
* JOINFILE - a program to read a number of files created by WinBack   *
*            SPLIT_FILE procedures and rejoin them into the original  *
* file. This version uses the available space in the jobs dataspace   *
* as the file buffer after allowing for the stack and variables that  *
* are used in the procedure.                                          *
*---------------------------------------------------------------------*
* Copyright - Norman Dunbar                                           *
*             December 1991 - February 1992.                          *
*=====================================================================*
* AMENDMENTS                                                          *
*                                                                     *
* 9th February 1992.                                                  *
* Section 0 not allowed by TOOLKIT 2 wildcard routines, section files *
* now start at 1 to avoid this problem.                               *
*                                                                     *
* T. Tebby strikes again, if a file has a section number = CODE('_')  *
* then this file is not copied back to hard disc either. The code has *
* been amended to miss out this file section number.                  *
*=====================================================================*

           size    10240             Dataspace of 10k

           get     'win2_trap1_hdr'
           get     'win2_trap2_hdr'
           get     'win2_trap3_hdr'
           get     'win2_errors_hdr'
           get     'win2_vectors_hdr'

*---------------------------------------------------------------------*
* General purpose equate values.                                      *
*---------------------------------------------------------------------*

stack      equ     $200                Minimum stack space required
linefeed   equ     $0A                 Linefeed character
timeout    equ    -$01                 TRAP # timeout value
me         equ    -$01                 This job's id (pseudo id that is)
name_len   equ     $2A                 Length of filename buffer (output)
black      equ     $00                 }
red        equ     $02                 } Mode 4 colours
green      equ     $04                 }
white      equ     $06                 }
buff_len   equ     $64                 100 byte buffer should suffice
head_len   equ     $40                 64 byte file header
underscore equ     '_'                 Miss out this section file
max_sect   equ     $60                 Maximum section number is 96 (!`)
max_name   equ     $27                 39 bytes is longest filename allowed
max_read   equ     32*1024-1           Longest IO_FSTRG or IO_SSTRG length
min_buff   equ     $0100               Minimum file buffer is 256 bytes

*---------------------------------------------------------------------*
* Offset from base of dataspace equates (base = A4).                  *
*---------------------------------------------------------------------*

console    equ     $00                 Console id
dest_file  equ     console+$04         Destination file id
dataspace  equ     dest_file+$04       Size of file buffer space
file_buff  equ     dataspace+$04       Start of output filename buffer
gen_buff   equ     file_buff+name_len+2 Start of general buffer
header     equ     gen_buff+buff_len+2  Start of file header buffer
section    equ     header+head_len     Section number
fdata      equ     section+2           Dataspace value for joined file
ftype      equ     fdata+4             File type (byte) for joined file
src_file   equ     ftype+2             Section file's id
to_do      equ     src_file+4          Bytes to be read from section file
dest_buff  equ     to_do+4             Buffer area for the file contents

end_vars   equ     dest_buff           Max space, used to calculate dataspace

*---------------------------------------------------------------------*
* This is where it all starts to happen.                              *
*---------------------------------------------------------------------*
start      bra.s   joinfile
           dc.l    0
           dc.w    $4afb

name       dc.w    end_name-name-2
           dc.b    'JOINFILE version 1.00'
end_name

align      ds.w     0                  Force word alignment

joinfile   adda.l   a6,a4              Make start of dataspace absolute
           adda.l   a6,a5              And end of dataspace

*----------------------------------------------------------------------*
* Read the current mode, if not mode 4 & monitor then set it.          *
*----------------------------------------------------------------------*
           move.l  a4,-(a7)            Corrupted by MT_DMODE
           moveq   #mt_dmode,d0        Prepare to read mode & display
           moveq   #-1,d1              Reading mode
           move.b  d1,d2               Reading display
           trap    #1                  Do it (no errors)
           tst.b   d1                  Mode 4 ?
           bne.s   set_mode            No, set mode 4
           tst.b   d2                  Monitor ?
           beq.s   mode_ok             Yes

set_mode   moveq   #mt_dmode,d0        Prepare to set mode
           moveq   #0,d1               Mode 4
           moveq   #0,d2               Monitor
           trap    #1                  Do it (no errors)

*----------------------------------------------------------------------*
* Mode is ok now, open a console window.                               *
*----------------------------------------------------------------------*
mode_ok    move.l  (a7)+,a4            Retrieve dataspace pointer
           move.w  ut_con,a2           Prepare to open a console
           lea.l   con_def,a1          Definition of it
           jsr     (a2)                Do it
           tst.l   d0                  Any errors ?
           beq.s   sign_on             No
           bsr     write_err0          Try to write the error to #0
           bra     job_end             And we cannot continue

*----------------------------------------------------------------------*
* Console is open, lets sign on.                                       *
*----------------------------------------------------------------------*
sign_on    move.l  a0,console(a4)      Store console id
           moveq   #red,d1             Red strip
           moveq   #sd_setst,d0
           bsr     fs_trap_3           Set strip
           moveq   #white,d1           Store new strip
           moveq   #sd_setin,d0        White ink
           bsr     fs_trap_3
           moveq   #0,d1               Width of 6 pixels
           moveq   #1,d2               Height of 20 pixels
           moveq   #sd_setsz,d0
           bsr     fs_trap_3           Set character size
           lea.l   name,a1             Task name
           bsr     write_text          Print it
           lea.l   copyright,a1        Copyright message
           bsr     write_text          Print it
           moveq   #0,d1               6 pixels wide
           moveq   #0,d2               10 pixels high
           moveq   #sd_setsz,d0
           bsr     fs_trap_3           Normal size again
           move.b  strip,d1            Get normal strip
           moveq   #sd_setst,d0
           bsr     fs_trap_3           Set strip
           move.b  ink,d1              Get normal ink
           moveq   #sd_setin,d0
           bsr     fs_trap_3           Set ink

*----------------------------------------------------------------------*
* Now calculate & print the file buffer size, if too small, bale out.  *
*----------------------------------------------------------------------*
calc_buff  move.l  a5,d0               End of dataspace
           sub.l   a4,d0               Minus start of dataspace
           sub.l   #stack,d0           Minus reserved stack space
           sub.l   #end_vars,d0        Minus reserved variable space
           sub.l   #min_buff,d0        Minus minimum buffer size
           addq.l  #1,d0               And make even
           bclr    #0,d0
           move.l  d0,dataspace(a4)    Store buffer size
           move.l  d0,-(a7)            Store for now
           bsr     write_mem           Print it
           move.l  (a7)+,d0            And restore again
           bge.s   main_loop           There is room

*----------------------------------------------------------------------*
* Buffer space too small, ask for more dataspace and bale out.         *
*----------------------------------------------------------------------*
too_small  lea.l   small,a1            Not enough buffer space message
           bsr     write_text          Print it
           move.w  #end_vars,d1        Variable space
           addi.w  #stack,d1           Plus stack space
           addi.w  #min_buff,d1        Plus minimum buffer
           bsr     print_d1            Print minimum space required
           beq.s   do_bytes            No error
           bsr     write_err0          Print error
           bra.s   bombed              And bomb out

do_bytes   lea.l   bytes,a1            "Bytes" message
           bsr     write_text          Print it

bombed     bra     any_key             Close up & bale out

*----------------------------------------------------------------------*
* The start of the main loop, ask for a filename, ENTER = quit, loop   *
* around until we have done all the user wants, getting each new file  *
* name & re-building it from its many sections.                        *
*----------------------------------------------------------------------*
main_loop  move.l  console(a4),a0      Get console id
           lea.l   prompt,a1           Prompt for filename
           bsr     write_text          Print it
           bsr     get_text            Accept a filename
           beq.s   check_end           No errors
           bra.s   main_loop           And try again

check_end  tst.w   d1                  Check if finished
           beq     any_key             Finished
           cmpi.w  #max_name,d1        Is name too long ?
           ble.s   move_dest           No

name_2_big lea.l   cant_open,a1        "Cannot open" message
           bsr     write_text          Print it
           lea.l   gen_buff(a4),a1     File name in question
           bsr     write_text          Print it
           lea.l   reason,a1           "Reason : " message
           bsr     write_text          Print it
           lea.l   too_big,a1          "Name too big" message
           bsr     write_text          Print it
           bra.s   main_loop           And try again

*----------------------------------------------------------------------*
* Move the filename to be joined from gen_buff to file_buff.           *
*----------------------------------------------------------------------*
move_dest  lea.l   gen_buff(a4),a1     Source pointer
           move.w  (a1)+,d0            How many bytes to move
           lea.l   file_buff(a4),a2    Destination pointer
           move.w  d0,(a2)+            Store length of file name
           subq.w  #1,d0               Adjust for debbie (!)

shift_byte move.b  (a1)+,(a2)+         Move a byte
           dbra    d0,shift_byte       And the rest

*----------------------------------------------------------------------*
* Now open the destination file, in overwrite mode.                    *
*----------------------------------------------------------------------*
open_dest  lea.l   file_buff(a4),a0    Pointer to file name
           moveq   #3,d3               New overwrite please
           bsr     open_file           Open destination file
           beq.s   dest_open           No errors

*----------------------------------------------------------------------*
* Destination file failed to open for some reason.                     *
*----------------------------------------------------------------------*
open_fail  move.l  d0,-(a7)            Store error for now
           lea.l   cant_open,a1        "Cannot open" message
           bsr     write_text          Print it
           lea.l   file_buff(a4),a1    Filename
           bsr     write_text          Print it
           lea.l   reason,a1           "reason :" message
           bsr     write_text          Print it
           move.l  (a7)+,d0            Retrieve error code
           bsr     write_err           Print error message
           bra     main_loop           And try again

*----------------------------------------------------------------------*
* Destination file is open.                                            *
*----------------------------------------------------------------------*
dest_open  move.l  a0,dest_file(a4)    Store dest channel id
           moveq   #1,d0               Section number
           move.w  d0,section(a4)      Store section number

*----------------------------------------------------------------------*
* Create the section file name in FILE_BUFF, however, due to a slight  *
* problem with TOOLKIT 2 routines such as WCOPY & WREN, some files are *
* not allowed. The main problems being with section 0 and section '_'  *
* as in the first case, the CHR$(0) is removed from the file name and  *
* in the case of the secon, the '_' is treated as a directory name and *
* not copied back from floppy. (Section now starts at 1 and not zero)  *                                  *
*----------------------------------------------------------------------*
make_name  move.w  section(a4),d0      Get the section number
           cmpi.w  #underscore,d0      WCOPY/WREN don't like this file
           beq     inc_sect            Miss out this section

check_max  cmpi.w  #max_sect,d0        Finished yet ?
           ble     not_done            No
           lea.l   all_sect,a1         "all sections" message
           bsr     write_text          Print it
           addi.w  #2,file_buff(a4)    Fiddle file length for DEST_HEAD
           bra     dest_head           Finished, go set header

not_done   lea.l   file_buff(a4),a1    Destination file name
           move.w  (a1),d1             Get length of dest file name
           move.b  #'!',2(a1,d1.w)     Store the marker
           move.b  d0,3(a1,d1.w)       And the section number
           move.l  a1,-(a7)            Save the pointer
           lea.l   process,a1          "Processing :" message
           bsr     write_text          Print it
           move.l  (a7),a1             File name (still on stack)
           move.w  (a1),d2             Length of filename
           bsr     print_sect          Print filename & section number

open_sect  move.l  (a7)+,a0            Pointer to filename & tidy stack
           addi.w  #2,(a0)             Include the marker & section number
           moveq   #1,d3               Old shared file
           bsr     open_file           Open the section file
           beq.s   sect_open           Yes
           cmpi.l  #err_nf,d0          Section not found ?
           bne.s   sect_err            No, some other error
           cmpi.w  #1,section(a4)      Did section 1 fail ERR_NF ?
           bne.s   not_sect_0          No, finished.
           lea.l   no_sects,a1         "no sections found" message
           bsr     write_text          Print it
           move.l  dest_file(a4),a0    Get destination channel id
           bsr     close_file          Close it
           lea.l   file_buff(a4),a0    Point to section name again
           subi.w  #2,(a0)             Make it into the destination filename
           moveq   #io_delet,d0        Deletion coming up
           moveq   #-1,d1              This job
           trap    #2                  Do it, but ignore errors

           bra     main_loop           Have another try

not_sect_0 lea.l   done,a1             "done" message
           bsr     write_text          Print it
           bra     dest_head           No more sections, file is joined

sect_err   move.l  d0,-(a7)            Save the error code
           lea.l   cant_open,a1        "Cannot open " message
           bsr     write_text          Print it
           lea.l   file_buff(a4),a1    File name
           move.w  (a1),d2             Name length
           subq.w  #2,d2               Minus marker & section bytes
           bsr     print_sect          Print file name
           lea.l   reason,a1           "Reason : " message
           bsr     write_text          Print it
           move.l  (a7)+,d0            Restore error code
           bsr     write_err           Print error text
           bra     dest_shut           Close destination file & try again

*----------------------------------------------------------------------*
* Section file is open for reading.                                    *
*----------------------------------------------------------------------*
sect_open  move.l  a0,src_file(a4)     Store section file id
           bsr     get_head            Read section header
           beq.s   head_read           No errors

*----------------------------------------------------------------------*
* Header has not been read from section file.                          *
*----------------------------------------------------------------------*
no_head    move.l  d0,-(a7)            Save error code
           lea.l   no_header,a1        "Cannot read header " message
           bsr     write_text          Print it
           lea.l   reason,a1           "Reason : " message
           bsr     write_text          Print it
           move.l  (a7)+,d0            Restore error code
           bsr     write_err           Print error text
           bsr     close_sect          Close source file
           bra     dest_shut           Close destination file & try again

*----------------------------------------------------------------------*
* Header has been read from section file, if this is the first section *
* we must retrieve the file type byte & the dataspace of the file as   *
* it was when it was originally split by WinBack. This is kept in the  *
* first section's file header.                                         *
*----------------------------------------------------------------------*
head_read  lea.l   header(a4),a1       Pointer to header buffer
           cmpi.w  #1,section(a4)      Is this section 1 ?
           bne.s   read_sect           No
           move.b  5(a1),ftype(a4)     Store file type byte
           move.l  6(a1),fdata(a4)     Store original dataspace

*----------------------------------------------------------------------*
* Complicated loop to read the section file into the buffer and write  *
* it out again, this part ensures that the calls to FILL_BUFF do not   *
* try to load more bytes than there is room in the file buffer.        *
*----------------------------------------------------------------------*
read_sect  move.l  (a1),d7             File length from header buffer
           beq     close_sect          Ignore zero length files
           move.l  d7,to_do(a4)        Store bytes to be read

sect_loop  move.l  to_do(a4),d7        Get number of remaining bytes to read
           move.l  dataspace(a4),d6    Get maximum length of buffer
           cmp.l   d6,d7               Check there is room in buffer
           ble.s   do_read             There is room for the remaining bytes
           move.l  d6,d7               The maximum allowed in the buffer

*----------------------------------------------------------------------*
* D7.L now hold either the number of bytes remaining in the section    *
* file to be read into the buffer, or the maximum number of bytes that *
* we can fit into our buffer. If any of these are greater than the 32K *
* that IO_FSTRG or IO_SSTRG can handle, don't worry, FILL_BUFF handles *
* that little problem.                                                 *
*----------------------------------------------------------------------*
do_read    move.l  d7,-(a7)            Save buffer length required
           bsr     fill_buff           Read required bytes into buffer
           beq.s   filled              No errors

oops       addq.l  #4,a7               Tidy stack, buffer length
           lea.l   no_read,a1          "Read error" message

come_here  move.l  d0,-(a7)            Save error code
           bsr     write_text          Print a message
           lea.l   reason,a1           "reason :" message
           bsr     write_text          Print it
           move.l  (a7)+,d0            Restore error code
           bsr     write_err           Print failure message
           lea.l   src_file(a4),a0     Section file id
           bsr     close_file          Close it
           bra     dest_shut           Tidy up & try again

*----------------------------------------------------------------------*
* Buffer has been filled, now write it out.                            *
*----------------------------------------------------------------------*
filled     move.l  (a7),d7             Restore buffer length (still on stack)
           bsr     write_buff          Write buffer to destination file
           beq.s   wrote_it            No errors
           lea.l   no_write,a1         "Write error" message
           bra.s   come_here           Print the rest of the message

wrote_it   move.l  (a7)+,d7            Finally get length off stack
           sub.l   d7,to_do(a4)        Adjust remaining bytes counter
           bne.s   sect_loop           And do some more if not finished

*----------------------------------------------------------------------*
* Section file has been written, close it.                             *
*----------------------------------------------------------------------*
close_sect move.l  src_file(a4),a0     Section file id
           bsr     close_file          Close it
           lea.l   finished,a1         "Finished" message
           bsr     write_text          Print it
           subi.w  #2,file_buff(a4)    Remove marker & section number

inc_sect   addi.w  #1,section(a4)      Increment section number
           bra     make_name           And the next one please

*----------------------------------------------------------------------*
* Set the header on the destination file, and we are finished.         *
*----------------------------------------------------------------------*
dest_head  move.l  dest_file(a4),a0    Get the channel id
           bsr     close_file          Close & flush file
           lea.l   file_buff(a4),a0    Point to the name again
           subi.w  #2,(a0)             Remove marker & section
           moveq   #0,d3               Old exclusive file
           bsr     open_file           Open it again
           beq.s   opened              No errors
           move.l  d0,-(a7)            Save error code
           lea.l   cant_open,a1        "Cannot open" message
           bsr     write_text          Print it
           lea.l   file_buff(a4),a1    File name
           move.w  (a1),d2             Length of filename + 2
           subq.w  #2,d2               Remove marker & section bytes
           bsr     print_sect          Print filename & section number
           lea.l   reason,a1           "reason" message
           bsr     write_text          Print it
           move.l  (a7)+,d0            Get error code
           bsr     write_err           Print error
           bra     main_loop           Try again

opened     move.l  a0,dest_file(a4)    Store id again
           bsr     get_head            Read in header
           beq.s   head_in             No errors
           move.l  d0,-(a7)            Save error code
           lea.l   no_header,a1        "Cannot read header" message
           bsr     write_text          Print it
           lea.l   reason,a1           "reason" message
           bsr     write_text          Print it
           move.l  (a7)+,d0            Get error code
           bsr     write_err           Print error
           bra.s   dest_shut           Close and try again

head_in    lea.l   header(a4),a1       Header buffer
           move.b  ftype(a4),5(a1)     Set the file type byte as original
           move.l  fdata(a4),6(a1)     Set dataspace as original
           moveq   #fs_heads,d0        Set the header
           bsr     fs_trap_3           Do it
           beq.s   dest_close          No errors, close & try again
           bsr     bad_set             Failed to set header

*----------------------------------------------------------------------*
* Finished with destination file so close it.                          *
*----------------------------------------------------------------------*
dest_close move.l  dest_file(a4),a0    Retrieve dest file id
           moveq   #fs_flush,d0        Make sure disc is written
           bsr     fs_trap_3           And flush header
           beq.s   dest_shut           No errors
           move.l  a0,-(a7)            Save dest file id
           bsr     bad_set             Failed to set header
           move.l  (a7)+,a0            Restore it for close

dest_shut  move.l  dest_file(a4),a0    In case called from afar...
           bsr     close_file          Close it
           bra     main_loop           Try for some more

*----------------------------------------------------------------------*
* Get a keypress from the user, close console channel & kill this job. *
*----------------------------------------------------------------------*
any_key    move.l  console(a4),a0      Console id
           lea.l   key_mess,a1         Press any key message
           move.w  ut_mtext,a2         Don't use WRITE_TEXT in case of errors
           jsr     (a2)                Print it
           moveq   #sd_cure,d0         Prepare to enable a cursor
           moveq   #timeout,d3         Take forever
           trap    #3                  Do it (ignore errors)
           moveq   #io_fbyte,d0        Fetch one byte
           trap    #3                  Go get it
           bsr     close_file          Close console channel

*----------------------------------------------------------------------*
* This job will self destruct in no time at all                        *
*----------------------------------------------------------------------*
job_end    moveq   #mt_frjob,d0        Job is about to die
           moveq   #me,d1              And it is this job
           trap    #1                  RIP (there is no return)
           bra.s   job_end             But I don't always believe that ...

*======================================================================*
* Various subrouines used by the program.                              *
*======================================================================*

*----------------------------------------------------------------------*
* Print some text to the console, pointer is in A1.L                   *
*----------------------------------------------------------------------*
write_text move.l  console(a4),a0      Console id
           move.w  ut_mtext,a2         Write text vector
           jsr     (a2)                Do it
           tst.l   d0                  Any errors ?
           bne.s   write_err0          Yes, try to print it
           rts                         Exit

*----------------------------------------------------------------------*
* Print the error in D0.L to the console.                              *
*----------------------------------------------------------------------*
write_err0 move.w  ut_err0,a2          Try to print error to #0
           jsr     (a2)                Do it (no errors)
           tst.l   d0                  Just in case, unset Z flag
           rts                         Return with D0 = ERR_??

*----------------------------------------------------------------------*
* Print the UNSIGNED value in D0.L to the console.                     *
*----------------------------------------------------------------------*
write_mem  lea.l   gen_buff(a4),a1     General work buffer
           movem.l d0-d3/a2,-(a7)      Working registers
           move.l  a1,-(a7)            And another
           addq.l  #2,a1               Point to text area of buffer
           lea.l   tens_table,a2       Powers of ten table
           moveq   #0,d1               Digit counter
           move.l  d0,d3               Value to be printed

next_digit move.l  (a2)+,d2            D2.L is current power of 10
           beq.s   all_done            But zero is end of table
           clr.b   d0                  D0.B is current digit

digit_loop sub.l   d2,d3               Subtract the current power of 10
           blt.s   buff_digit          Too far
           addq.b  #1,d0               Increase current digit
           bra.s   digit_loop          And try again

buff_digit add.l   d2,d3               Correct for the overflow
           tst.b   d0                  Is this a zero ?
           bne.s   not_a_zero          No
           tst.w   d1                  Yes, is it a leading zero ?
           beq.s   next_digit          Yes, ignore it

not_a_zero addi.b  #'0',d0             Convert to ASCII
           move.b  d0,(a1)+            Store in buffer
           addq.w  #1,d1               Increment total digits
           bra.s   next_digit          And do the rest

*----------------------------------------------------------------------*
* Check for a result of zero. In this case force a '0' to be printed   *
*----------------------------------------------------------------------*
all_done   tst.w   d1                  Any digits found ?
           bne.s   not_zero            yes
           move.b  #'0',(a1)           Store a zero
           moveq.w #1,d1               And set the count

not_zero   move.l  (a7)+,a1            Start of buffer
           move.w  d1,(a1)             Store string length
           movem.l (a7)+,d0-d3/a2      Restore registers
           move.l  a1,-(a7)            Buffer on stack again
           lea.l   data_mess,a1        Start of message
           bsr     write_text          Print it
           move.l  (a7)+,a1            Start of buffer size
           bsr     write_text          Print it
           lea.l   bytes,a1            "bytes" message
           bsr     write_text          Print it
           rts                         Exit

*----------------------------------------------------------------------*
* Accept some text from the console.                                   *
*----------------------------------------------------------------------*
get_text   moveq   #buff_len,d2        Maximum buffer length
           lea.l   gen_buff(a4),a1     General buffer
           move.l  console(a4),a0      Console id
           move.l  a1,-(a7)            Store buffer address
           addq.l  #2,a1               Point to text area of buffer
           moveq   #io_fline,d0        Prepare to get some text
           moveq   #timeout,d3         Dont ask ...
           trap    #3                  Do it
           move.l  (a7)+,a1            Word length address
           subq.w  #1,d1               Subtract ENTER character
           move.w  d1,(a1)             Store it
           tst.l   d0                  Check for errors
           rts

*----------------------------------------------------------------------*
* Print the error in D0.L to the console.                              *
*----------------------------------------------------------------------*
write_err  move.w  ut_err,a2           Prepare to print error and linefeed
           move.l  console(a4),a0      Console id
           jsr     (a2)                Print error
           rts

*----------------------------------------------------------------------*
* Close a file whose channel id is in A0.L                             *
*----------------------------------------------------------------------*
close_file moveq   #io_close,d0        Prepare to close a file
           trap    #2                  Do it
           moveq   #0,d0               Never fails !
           rts                         Exit

*----------------------------------------------------------------------*
* Open a file, type is in D3, A0 points to name.                       *
*----------------------------------------------------------------------*
open_file  moveq   #io_open,d0         Prepare to open it
           moveq   #me,d1              This job owns the file
           trap    #2                  Open it
           tst.l   d0                  Ok ?
           rts                         Return with error in D0 & Zero flag

*----------------------------------------------------------------------*
* Read the header of the file whose id is in A0.                       *
*----------------------------------------------------------------------*
get_head   moveq   #fs_headr,d0        Prepare to read header
           lea.l   header(a4),a1       Header buffer
           moveq   #head_len,d2        Size of buffer


*----------------------------------------------------------------------*
* Call here to do an FS_XXXX trap with D0, A0, A1 etc set by caller.   *
*----------------------------------------------------------------------*
fs_trap_3  moveq   #timeout,d3         Don't ask ...
           trap    #3                  Do it
           tst.l   d0                  Ok ?
           rts                         Return error code & Z flag

*----------------------------------------------------------------------*
* Cannot set file header for a reason which will become clear.         *
*----------------------------------------------------------------------*
bad_set    move.l  d0,-(a7)            Save error code
           lea.l   no_set,a1           "Cannot set header" message
           bsr     write_text          Print it
           lea.l   reason,a1           "reason" message
           bsr     write_text          Print it
           move.l  (a7)+,d0            Get error code
           bsr     write_err           Print error
           rts

*----------------------------------------------------------------------*
* Fill a buffer with bytes from a file. If the buffer is longer than   *
* IO_FSTRG can handle in one go, do it in chunks. The amount of bytes  *
* to be read in is held in D7.L                                        *
*----------------------------------------------------------------------*
fill_buff  lea.l   dest_buff(a4),a1    Where the buffer starts
           move.l  src_file(a4),a0     File to be read
           moveq   #timeout,d3
           moveq   #0,d1               Clear top of D1
           moveq   #0,d2               And D2

loop_fill  move.w  d7,d2               Assume length is a word, but...
           cmpi.l  #max_read,d7        Check size as a long
           ble.s   read_it             Yes, length is ok
           move.w  #max_read,d2        Only read the max allowed length

read_it    moveq   #io_fstrg,d0        Prepare to read a string of bytes
           trap    #3                  Do it
           tst.l   d0                  Ok ?
           beq.s   read_ok             Yes
           rts                         Return with Z unset for errors

read_ok    sub.l   d1,d7               Subtract (long) the bytes read (word)
           tst.l   d7                  Finished yet ?
           bne.s   loop_fill           No, most registers are still ok
           rts                         Return with Z set for all ok

*----------------------------------------------------------------------*
* Fill a file with bytes from a buffer. If the buffer is longer than   *
* IO_SSTRG can handle in one go, do it in chunks. The amount of bytes  *
* to be written in is held in D7.L                                     *
*----------------------------------------------------------------------*
write_buff lea.l   dest_buff(a4),a1    Where the buffer starts
           move.l  dest_file(a4),a0    Which file to write
           moveq   #timeout,d3
           moveq   #0,d1               Clear top word
           moveq   #0,d2               Ditto

loop_write move.w  d7,d2               Pretend length is a word, but ...
           cmpi.l  #max_read,d7        Check length
           ble.s   write_it            Length is a word
           move.w  #max_read,d2        Maximum amount allowed to write

write_it   moveq   #io_sstrg,d0        Prepare to write a string
           trap    #3                  Do it
           tst.l   d0                  Ok ?
           beq.s   write_ok            Yes
           rts                         Return with Z unset for errors

write_ok   sub.l   d1,d7               Subtract (long) the bytes send (word)
           tst.l   d7                  Finished yet ?
           bne.s   loop_write          No
           rts                         Return with Z set for all ok

*----------------------------------------------------------------------*
* Print the contents of D1.W as ASCII.                                 *
*----------------------------------------------------------------------*
print_d1   move.l  console(a4),a0      Console channel id
           move.w  ut_mint,a2          Print D1.W as ASCII text
           jsr     (a2)                And do it
           tst.l   d0                  Hopefully it worked...
           rts                         Set flags anyway

*----------------------------------------------------------------------*
* Print the name of a section file plus its section number, Pointer to *
* name is in A1.L and length of name is in D2.W                        *
*----------------------------------------------------------------------*
print_sect move.l  console(a4),a0      Console channel id
           moveq   #io_sstrg,d0        Not using WRITE_TEXT
           addq.l  #2,a1               Point past word count
           bsr     fs_trap_3           Print filename
           bne.s   print_exit          Oops...
           lea.l   sect_text,a1        "Section" message
           bsr     write_text          Print it
           bne.s   print_exit          Oops...
           move.w  section(a4),d1      Get current section number
           bsr     print_d1            Print it

print_exit rts                         Exit with appropriate error code

*======================================================================*
* Various messages & prompts used by the program.                      *
*======================================================================*

copyright  dc.w    end_copy-copyright-2
           dc.b    '          '
           dc.b    'Copyright - Norman Dunbar, Dec 91 - Feb 92'
           dc.b    linefeed
end_copy

small      dc.w    end_small-small-2
           dc.b    linefeed
           dc.b    'There is not enough dataspace for a file buffer'
           dc.b    linefeed
           dc.b    'Please run DATASPACE_EXE to increase it.'
           dc.b    linefeed
           dc.b    'The minimum dataspace required is '
end_small

data_mess  dc.w    end_data-data_mess-2
           dc.b    linefeed
           dc.b    'File buffer is '
end_data

bytes      dc.w    end_bytes-bytes-2
           dc.b    ' bytes.'
           dc.b    linefeed
end_bytes

key_mess   dc.w    end_key-key_mess-2
           dc.b    linefeed
           dc.b    'Press any key to kill JOINFILE .... '
end_key

cant_open  dc.w    end_cant-cant_open-2
           dc.b    linefeed
           dc.b    'Cannot open '
end_cant

reason     dc.w    end_reason-reason-2
           dc.b    ', reason : '
end_reason

prompt     dc.w    end_prompt-prompt-2
           dc.b    linefeed
           dc.b    'Enter filename to be joined from its many sections'
           dc.b    linefeed
           dc.b    'or ENTER only to quit : '
end_prompt

too_big    dc.w    end_2_big-too_big-2
           dc.b    linefeed
           dc.b    'Name greater than 39 characters.'
           dc.b    linefeed
end_2_big

process    dc.w    end_process-process-2
           dc.b    'Processing : '
end_process

sect_text  dc.w    end_sect-sect_text-2
           dc.b    ' section '
end_sect

no_header  dc.w    end_no_head-no_header-2
           dc.b    linefeed
           dc.b    'Cannot read header,'
end_no_head

finished   dc.w    end_finished-finished-2
           dc.b    ', finished.'
           dc.b    linefeed
end_finished

done       dc.w    end_done-done-2
           dc.b    ', done.'
           dc.b    linefeed
end_done

no_sects   dc.w    end_sects-no_sects-2
           dc.b    ', no sections found.'
           dc.b    linefeed
end_sects

all_sect   dc.w    end_all_sect-all_sect-2
           dc.b    linefeed
           dc.b    'All sections processed.'
           dc.b    linefeed
end_all_sect

no_set     dc.w    end_no_set-no_set-2
           dc.b    linefeed
           dc.b    'Cannot set header,'
end_no_set

no_read    dc.w    end_no_read-no_read-2
           dc.b    linefeed
           dc.b    'read error'
end_no_read

no_write   dc.w    end_no_write-no_write-2
           dc.b    linefeed
           dc.b    'write error'
end_no_write


*----------------------------------------------------------------------*
* Mode 4, monitor console definition block.                            *
*----------------------------------------------------------------------*

           ds.w    0                   Align again

con_def    dc.b    $D0                 Border colour = red/black
           dc.b    $02                 Border width
strip      dc.b    green               Paper colour
ink        dc.b    black               Ink colour
           dc.w    $01BE               Width = 442
           dc.w    $0078               Height = 120
           dc.w    $0020               X position = 32
           dc.w    $0010               Y position = 16

*----------------------------------------------------------------------*
* A table of all powers of ten, from 9 to 0. This corresponds to the   *
* values used when converting an UNSIGNED long word to ASCII.          *
* 2^31 = 2,147,483,648                                                 *
* 10^9 = 1,000,000,000 so is a big enough 'highest' power to use       *
*----------------------------------------------------------------------*

tens_table dc.l    1000000000          10 ^ 9
           dc.l    100000000           10 ^ 8
           dc.l    10000000            10 ^ 7
           dc.l    1000000             10 ^ 6
           dc.l    100000              10 ^ 5
           dc.l    10000               10 ^ 4
           dc.l    1000                10 ^ 3
           dc.l    100                 10 ^ 2
           dc.l    10                  10 ^ 1
           dc.l    1                   10 ^ 0
           dc.l    0                   Table end marker

           END



